# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.fti import DexterityFTI
from transaction import commit

import unittest


class LuogoBehaviorIndexerFunctionalTest(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_FUNCTIONAL_TESTING
    maxDiff = None

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.catalog = api.portal.get_tool("portal_catalog")
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        fti = DexterityFTI("venueitem")
        self.portal.portal_types._setObject("venueitem", fti)
        fti.klass = "plone.dexterity.content.Item"
        fti.behaviors = (
            "plone.app.content.interfaces.INameFromTitle",
            "plone.app.dexterity.behaviors.metadata.IBasic",
            "plone.app.dexterity.behaviors.metadata.ICategorization",
            "collective.geolocationbehavior.geolocation.IGeolocatable",
            "design.plone.contenttypes.behavior.additional_help_infos",
            "design.plone.contenttypes.behavior.argomenti",
            "plone.leadimage",
            "volto.preview_image",
            "collective.address.behaviors.IAddress",
            "design.plone.contenttypes.behavior.luogo",
            "plone.textindexer",
        )
        self.fti = fti

        self.portal.invokeFactory("venueitem", id="venue", title="venue")
        self.venue = self.portal.venue
        commit()

    def test_luogo_behavior_fields_inexed_for_venue(self):
        # Non sembra deterministico il testing delle cose indicizzate con
        # plone.textindexer. Per ora togliamo. Poi se capiamo
        # come gestire lo rimetteremo.
        return
        self.assertTrue(True)
        res = api.content.find(UID=self.venue.UID())
        rid = res[0].getRID()

        self.assertEqual(
            self.catalog.getIndexDataForRID(rid)["SearchableText"], ["venue"]
        )

        self.venue.quartiere = "quartiere"
        self.venue.circoscrizione = "Nord/ovest"
        self.venue.descrizione_breve = {
            "blocks": {"123456": {"searchableText": "breve"}}
        }
        self.venue.orario_pubblico = {
            "blocks": {"123456": {"searchableText": "orario"}}
        }
        self.venue.reindexObject(idxs="SearchableText")
        commit()

        index_data = self.catalog.getIndexDataForRID(rid)
        self.assertIn("quartiere", index_data["SearchableText"])
        self.assertIn("nord", index_data["SearchableText"])
        self.assertIn("ovest", index_data["SearchableText"])
        self.assertIn("breve", index_data["SearchableText"])
        self.assertIn("orario", index_data["SearchableText"])
        self.assertEqual(
            index_data["SearchableText"],
            ["venue", "quartiere", "nord", "ovest", "breve", "orario"],
        )

        res = api.content.find(SearchableText="breve")
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0].UID, self.venue.UID())
